# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2018-2025, by Samuel Williams.

require_relative "localhost/version"
require_relative "localhost/authority"
require_relative "localhost/system"

# @namespace
module Localhost
end
